#!/usr/bin/env python
"""
synopsis:
    Display links (<a>) in a page.  Show URLs and, optionally,
    show the label.  Optionally, show links in linked to pages to
    a specified depth.
    Requires BeautifulSoup -- https://pypi.python.org/pypi/BeautifulSoup
examples:
    python showlinks.py -d 2 "http://www.davekuhlman.org"
    python showlinks.py -d 2 "http://www.davekuhlman.org"
    python showlinks.py -x ".jpg" -d 2 "http://www.davekuhlman.org"
notes:
    Can also pass in options with the SHOWLINKS_ARGS environment
    variable, for example (on Linux):
        $ export SHOWLINKS_ARGS="-d 2"
"""


import sys
import os
import argparse
import urllib2
import shlex
# Import BeautifulSoup.
import bs4


# Ignore the following extensions.
# Also see command line flag -x (--additional-extensions).
Skip_extensions = [
    '.bz2',
    '.gz',
    '.gzip',
    '.odt',
    '.pdf',
    '.tar',
    '.xz',
    '.zip',
]


def show_links(args):
    """Show the links in a page."""
    rooturl = args.rooturl
    page = make_bs_page(rooturl, args)
    level = 0
    show_links_recursive(page, level, args)
    if args.totals is not None:
        args.outfile.write('----------\nTotal links: {}\n'.format(args.totals))


def show_links_recursive(page, level, args):
    """Recursively show the links in a page and linked to pages.

    Descend to a specified depth.
    """
    if page is None:
        return
    wrt = args.outfile.write
    urls = page.find_all('a')
    for url in urls:
        urlhref = url.get('href', '')
        if not urlhref.startswith('http://'):
            continue
        if args.totals is not None:
            args.totals += 1
        level1 = level + 1
        wrt('{}url: {}\n'.format('    ' * level, urlhref.encode('utf-8')))
        if args.labels:
            label = url.get_text()
            label = label.encode('utf-8')
            wrt('    {}label: "{}"\n'.format('    ' * level, label))
        if level1 >= args.depth:
            continue
        page = make_bs_page(urlhref, args)
        show_links_recursive(page, level1, args)


def make_bs_page(url, args):
    """Create a BeautifulSoup page object from a URL."""
    if not screen_extensions(url):
        return None
    page = None
    try:
        infile = urllib2.urlopen(url)
    except urllib2.URLError:
        infile = None
    if infile is not None:
        page = bs4.BeautifulSoup(infile, args.parser)
        infile.close()
    return page


def screen_extensions(url):
    """Return true unless the URL has a file extension we want to skip."""
    base, ext = os.path.splitext(url)
    if ext and ext not in Skip_extensions:
        return True
    return False


def open_file(args):
    """Open/create a file if it does not exist or the -f option was given."""
    if not args.force:
        if os.path.exists(args.outfile):
            sys.exit('\noutput file exists.  use -f/--force to overwrite.\n')
    outfile = open(args.outfile, 'w')
    return outfile


def make_arg_parser(parser=None):
    if parser is None:
        parser = argparse.ArgumentParser(
            description=__doc__,
            formatter_class=argparse.RawDescriptionHelpFormatter,)
    parser.add_argument(
        'rooturl', type=str,
        help='root url')
    parser.add_argument(
        '-d', '--max-depth', dest='depth', type=int,
        action='store', default=1,
        help='maximum depth to recurse.  default: 1')
    parser.add_argument(
        '-v', '--verbose', dest='verbose', action='store_true',
        default=False,
        help="print additional info and actions.  default: False")
    parser.add_argument(
        '-t', '--totals', dest='totals', action='store_true',
        default=False,
        help="print total number of links found.  default: False")
    parser.add_argument(
        '-p', '--parser', dest='parser', action='store',
        type=str, default='html.parser',
        help='parser to be used, e.g. "lxml".  '
             'default: BeautifulSoup default parser')
    parser.add_argument(
        '-x', '--additional-extensions', dest='extensions', action='store',
        type=str, default='',
        help='additional file name extensions to be ignored, '
             'space separated.  Example: ".xxx .yyy".  '
             'See global variable Skip_extensions.  '
             'default: ""')
    parser.add_argument(
        '-l', '--labels', dest='labels', action='store_true',
        default=False,
        help="also show label of link.  default: False")
    parser.add_argument(
        '-o', '--outfile', dest='outfile', type=str, action='store',
        default='', metavar='outfilename',
        help='file to write results to.  default: sys.stdout')
    parser.add_argument(
        '-f', '--force', dest='force', action='store_true',
        default=False,
        help='force overwrite of output file.')
    return parser


def main(argv=None):
    try:
        if argv is None:
            # Look at the SHOWLINKS_ARGS environment variable for
            # more arguments.
            env_args = shlex.split(os.getenv('SHOWLINKS_ARGS', ''))
            argv = [sys.argv[0]] + env_args + sys.argv[1:]
        parser = make_arg_parser()
        args = parser.parse_args(argv[1:])
        Skip_extensions.extend(args.extensions.split())
        if args.outfile:
            args.outfile = open_file(args)
        else:
            args.outfile = sys.stdout
        if args.totals:
            args.totals = 0
        else:
            args.totals = None
        show_links(args)
    except KeyboardInterrupt:
        raise SystemExit(0)
    except IOError as e:
        if 'Broken pipe' in str(e):
            # The user is probably piping to a pager like less(1)
            # and has exited it. Just exit.
            raise SystemExit(0)
        raise


if __name__ == '__main__':
    #import ipdb; ipdb.set_trace()
    main()
